package org.bitbucket.bradleysmithllc.etlunit.feature.database.db;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.google.gson.stream.JsonWriter;
import org.bitbucket.bradleysmithllc.etlunit.util.StringUtils;

import java.io.IOException;
import java.util.*;

public class CatalogImpl implements Catalog
{
	private String name;
	private final Database database;

	private final List<Schema> schemaList = new ArrayList<Schema>();
	private final Map<String, Schema> schemaMap = new HashMap<String, Schema>();

	public CatalogImpl(Database database, String name) {
		this.name = name;
		this.database = database;
	}

	public CatalogImpl(Database database) {
		this.database = database;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public Database getDatabase() {
		return database;
	}

	@Override
	public List<Schema> getSchemas() {
		return Collections.unmodifiableList(schemaList);
	}

	@Override
	public Map<String, Schema> getSchemaMap() {
		return Collections.unmodifiableMap(schemaMap);
	}

	@Override
	public Schema getSchema(String name) {
		String safeNullableUppercase = StringUtils.getSafeNullableUppercase(name);
		Schema s = schemaMap.get(safeNullableUppercase);

		if (s == null)
		{
			return addSchema(new SchemaImpl(this, safeNullableUppercase));
		}

		return s;
	}

	public Schema addSchema(Schema schema)
	{
		String safeNullableUppercase = StringUtils.getSafeNullableUppercase(schema.getName());
		if (schemaMap.containsKey(safeNullableUppercase))
		{
			throw new IllegalArgumentException("Schema name [" + safeNullableUppercase + "] already added to catalog");
		}

		schemaList.add(schema);
		schemaMap.put(safeNullableUppercase, schema);

		touch();

		return schema;
	}

	public String toString()
	{
		return database.toString() + "." + (name == null ? "[DEFAULT]" : name.toString());
	}

	@Override
	public void toJson(JsonWriter writer) throws IOException {
		writer.name("name");

		if (name == null)
		{
			writer.nullValue();
		}
		else
		{
			writer.value(name);
		}

		writer.name("schemas").beginArray();

		for (Schema table : schemaList)
		{
			writer.beginObject();
			table.toJson(writer);
			writer.endObject();
		}

		writer.endArray();
	}

	@Override
	public void fromJson(JsonNode node) {
		JsonNode jsonNode = node.get("name");

		if (!jsonNode.isNull())
		{
			name = jsonNode.asText();
		}

		JsonNode tables = node.get("schemas");

		if (tables != null && !tables.isNull())
		{
			ArrayNode nodeList = (ArrayNode) tables;

			Iterator<JsonNode> it = nodeList.iterator();

			while (it.hasNext())
			{
				JsonNode tnode = it.next();

				SchemaImpl timpl = new SchemaImpl(this);

				timpl.fromJson(tnode);

				addSchema(timpl);
			}
		}
	}

	public void touch()
	{
		if (database != null)
		{
			database.touch();
		}
	}
}